@@ -7,12 +7,15 @@ import json
 import numpy as np
 import pandas as pd
 from astropy.table import Table as AstropyTable
+from astropy.units.quantity import Quantity
+from astropy.coordinates import SkyCoord
 from sqlalchemy.orm import sessionmaker
 from sqlalchemy.orm.query import Query
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.engine import Engine
 from sqlalchemy import event, create_engine, Table
 from sqlalchemy import or_, and_
+from tqdm import tqdm
 from . import REFERENCE_TABLES, PRIMARY_TABLE, PRIMARY_TABLE_KEY, FOREIGN_KEY
 from .utils import json_serializer, get_simbad_names, deprecated_alias, datetime_json_parser
 from .spectra import load_spectrum
@@ -174,7 +177,7 @@ def set_sqlite():
         cursor.close()
 
 
-def create_database(connection_string):
+def create_database(connection_string, drop_tables=False):
     """
     Create a database from a schema that utilizes the `astrodbkit2.astrodb.Base` class.
     Some databases, eg Postgres, must already exist but any tables should be dropped.
@@ -183,10 +186,13 @@ def create_database(connection_string):
     ----------
     connection_string : str
         Connection string to database
+    drop_tables : bool
+        Flag to drop existing tables. This is needed when the schema changes. (Default: False)
     """
 
     session, base, engine = load_connection(connection_string, base=Base)
-    # base.metadata.drop_all()  # drop all the tables
+    if drop_tables:
+        base.metadata.drop_all()
     base.metadata.create_all()  # this explicitly creates the database
 
 
@@ -411,7 +417,7 @@ class Database:
     @deprecated_alias(format='fmt')
     def search_object(self, name, output_table=None, resolve_simbad=False,
                       table_names={'Sources': ['source', 'shortname'], 'Names': ['other_name']},
-                      fmt='default', fuzzy_search=True, verbose=True):
+                      fmt='table', fuzzy_search=True, verbose=True):
         """
         Query the database for the object specified. By default will return the primary table,
         but this can be specified. Users can also request to resolve the object name via Simbad and query against
@@ -429,7 +435,7 @@ class Database:
             Dictionary of tables to search for name information. Should be of the form table name: column name list.
             Default: {'Sources': ['source', 'shortname'], 'Names': 'other_name'}
         fmt : str
-            Format to return results in (pandas, astropy/table, default)
+            Format to return results in (pandas, astropy/table, default). Default is astropy table
         fuzzy_search : bool
             Flag to perform partial searches on provided names (default: True)
         verbose : bool
@@ -452,7 +458,7 @@ class Database:
 
         # Query Simbad to get additional names and join them to list to search
         if resolve_simbad:
-            simbad_names = get_simbad_names(name)
+            simbad_names = get_simbad_names(name, verbose=verbose)
             name = list(set(simbad_names + [name]))
             if verbose:
                 print(f'Including Simbad names, searching for: {name}')
@@ -524,6 +530,86 @@ class Database:
 
         return self._handle_format(temp, fmt)
 
+    def query_region(self, target_coords, radius=Quantity(10, unit='arcsec'), output_table=None, fmt='table',
+                    coordinate_table=None, ra_col='ra', dec_col='dec', frame='icrs', unit='deg'):
+        """
+        Perform a cone search of the given coordinates and return the specified output table.
+
+        Parameters
+        ----------
+        target_coords : SkyCoord
+            Astropy SkyCoord object of coordinates to search around
+        radius : Quantity or float
+            Radius as an astropy Quantity object in which to search for objects.
+            If not a Quantity will convert to one assuming units are arcseconds. Default: 10 arcseconds
+        output_table : str
+            Name of table to match. Default: primary table (eg, Sources)
+        fmt : str
+            Format to return results in (pandas, astropy/table, default). Default is astropy table
+        coordinate_table : str
+            Table to use for coordinates. Default: primary table (eg, Sources)
+        ra_col : str
+            Name of column to use for RA values. Default: ra
+        dec_col : str
+            Name of column to use for Dec values. Default: dec
+        frame : str
+            Coordinate frame for objects in the database. Default: icrs
+        unit : str or tuple of Unit or str
+            Unit of ra/dec (or equivalent) in database. Default: deg
+
+        Returns
+        -------
+        List of SQLAlchemy results
+        """
+
+        # Set table to output and verify it exists
+        if output_table is None:
+            output_table = self._primary_table
+        if output_table not in self.metadata.tables:
+            raise RuntimeError(f'Table {output_table} is not in the database')
+
+        # Radius conversion
+        if not isinstance(radius, Quantity):
+            radius = Quantity(radius, unit='arcsec')
+
+        # Get the column name to use for matching
+        match_column = self._foreign_key
+        if output_table == self._primary_table:
+            match_column = self._primary_table_key
+
+        # Grab the specified coordinate table (Sources by default) to construct SkyCoord objects
+        if coordinate_table is None:
+            coordinate_table = self._primary_table
+        if coordinate_table not in self.metadata.tables:
+            raise RuntimeError(f'Table {coordinate_table} is not in the database')
+        coordinate_match_column = self._foreign_key
+        if coordinate_table == self._primary_table:
+            coordinate_match_column = self._primary_table_key
+
+        # This is adapted from the original astrodbkit code
+        df = self.query(self.metadata.tables[coordinate_table]).pandas()
+        df[['ra', 'dec']] = df[[ra_col, dec_col]].apply(pd.to_numeric)  # convert everything to floats
+        mask = df['ra'].isnull()
+        df = df[~mask]
+
+        # Native use of astropy SkyCoord objects here
+        coord_list = SkyCoord(df['ra'].tolist(), df['dec'].tolist(), frame=frame, unit=unit)
+        sep_list = coord_list.separation(target_coords)  # sky separations for each db object against target position
+        good = sep_list <= radius
+
+        if sum(good) > 0:
+            matched_list = df[coordinate_match_column][good]
+        else:
+            matched_list = []
+
+        # Join the matched sources with the desired table
+        temp = self.query(self.metadata.tables[output_table]). \
+            filter(self.metadata.tables[output_table].columns[match_column].in_(matched_list)). \
+            all()
+        results = self._handle_format(temp, fmt)
+
+        return results
+
     # Object output methods
     def save_json(self, name, directory):
         """
@@ -550,6 +636,24 @@ class Database:
         with open(os.path.join(directory, filename), 'w') as f:
             f.write(json.dumps(data, indent=4, default=json_serializer))
 
+    def save_reference_table(self, table, directory):
+        """
+
+        Parameters
+        ----------
+        table : str
+            Name of reference table to output
+        directory : str
+            Name of directory in which to save the output JSON
+        """
+
+        results = self.session.query(self.metadata.tables[table]).all()
+        data = [row._asdict() for row in results]
+        filename = table + '.json'
+        if len(data) > 0:
+            with open(os.path.join(directory, filename), 'w') as f:
+                f.write(json.dumps(data, indent=4, default=json_serializer))
+
     def save_database(self, directory, clear_first=True):
         """
         Output contents of the database into the specified directory as JSON files.
@@ -576,15 +680,10 @@ class Database:
             if table not in self.metadata.tables.keys():
                 continue
 
-            results = self.session.query(self.metadata.tables[table]).all()
-            data = [row._asdict() for row in results]
-            filename = table + '.json'
-            if len(data) > 0:
-                with open(os.path.join(directory, filename), 'w') as f:
-                    f.write(json.dumps(data, indent=4, default=json_serializer))
+            self.save_reference_table(table, directory)
 
         # Output primary objects
-        for row in self.query(self.metadata.tables[self._primary_table]):
+        for row in tqdm(self.query(self.metadata.tables[self._primary_table])):
             self.save_json(row, directory)
 
     # Object input methods
@@ -636,7 +735,7 @@ class Database:
         # Load into specified table
         self.metadata.tables[table].insert().execute(data)
 
-    def load_table(self, table, directory):
+    def load_table(self, table, directory, verbose=False):
         """
         Load a reference table to the database, expects there to be a file of the form [table].json
 
@@ -646,6 +745,8 @@ class Database:
             Name of table to load. Table must already exist in the schema.
         directory : str
             Name of directory containing the JSON file
+        verbose : bool
+            Flag to enable diagnostic messages
         """
 
         filename = os.path.join(directory, table+'.json')
@@ -654,7 +755,7 @@ class Database:
                 data = json.load(f)
                 self.metadata.tables[table].insert().execute(data)
         else:
-            print(f'{table}.json not found.')
+            if verbose: print(f'{table}.json not found.')
 
     def load_json(self, filename):
         """
@@ -705,15 +806,19 @@ class Database:
         # Load reference tables first
         for table in self._reference_tables:
             if verbose: print(f'Loading {table} table')
-            self.load_table(table, directory)
+            self.load_table(table, directory, verbose=verbose)
 
         # Load object data
         if verbose: print('Loading object tables')
-        for file in os.listdir(directory):
+        for file in tqdm(os.listdir(directory)):
             # Skip reference tables
             core_name = file.replace('.json', '')
             if core_name in self._reference_tables:
                 continue
 
+            # Skip non-JSON files or hidden files
+            if not file.endswith('.json') or file.startswith('.'):
+                continue
+
             self.load_json(os.path.join(directory, file))
 
